@@ -19,8 +19,17 @@
 import signal
 import threading
 
+from time import sleep
+from inspect import isclass
+from itertools import count
 from functools import wraps
 from types import MethodType
+from traceback import print_exc
+
+try:  # Python 2
+    from Queue import Queue
+except:  # Python 3
+    from queue import Queue
 
 
 # Pool states
@@ -40,6 +49,11 @@ class PebbleError(Exception):
     pass
 
 
+class PoolError(PebbleError):
+    """Raised if an error occurred within the Pool."""
+    pass
+
+
 class TaskCancelled(PebbleError):
     """Raised if get is called on a cancelled task."""
     pass
@@ -319,3 +333,124 @@ class Task(object):
                 self._ready = True
                 self._results = results
                 self._task_ready.notify_all()
+
+
+class PoolContext(object):
+    """Pool's Context.
+
+    Wraps the Pool's state.
+
+    """
+    def __init__(self, queue, queueargs, initializer, initargs,
+                 workers, limit):
+        self.state = CREATED
+        self.pool = {}  # {tid/pid: Thread/Process}
+        self.tasks = {}  # {Task.number: Task}
+        self.managers = None  # threads managing the Pool
+        self.initializer = initializer
+        self.initargs = initargs
+        self.worker_number = workers
+        self.worker_limit = limit
+        if queue is not None:
+            if isclass(queue):
+                self.queue = queue(*queueargs)
+            else:
+                raise ValueError("Queue must be Class")
+        else:
+            self.queue = Queue()
+
+    def join(self, timeout):
+        """Joins pool's workers."""
+        while len(self.pool) > 0 and (timeout is None or timeout > 0):
+            for identifier, worker in list(self.pool.items()):
+                worker.join(timeout is not None and 0.1 or None)
+                if not worker.is_alive():
+                    self.pool.pop(identifier)
+
+            if timeout is not None:
+                timeout = timeout - (len(self.pool) / 10.0)
+
+        if len(self.pool) > 0:
+            raise TimeoutError('Workers are still running')
+
+    def task_done(self, task, results):
+        task._set(results)
+
+        if task._callback is not None:
+            try:
+                task._callback(task)
+            except Exception:
+                print_exc()
+                self.state = ERROR
+
+        self.queue.task_done()
+
+
+class BasePool(object):
+    def __init__(self):
+        self._counter = count()
+        self._context = None
+        self._managers = ()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        self.close()
+        self.join()
+
+    @property
+    def active(self):
+        return self._context.state == RUNNING and True or False
+
+    def _state(self):
+        """Updates Pool's state."""
+        if self._context.state == CREATED:
+            self._start()
+        else:
+            for manager in self._managers:
+                if not manager.is_alive():
+                    self._context.state = ERROR
+
+    def _schedule(self, task):
+        """Schedules *Task* into the Pool."""
+        self._state()
+
+        if self._context.state == ERROR:
+            raise PoolError('Unexpected error within the Pool')
+        elif self._context.state != RUNNING:
+            raise RuntimeError('The Pool is not running')
+
+        self._context.queue.put(task)
+
+        return task
+
+    def close(self):
+        """Closes the pool.
+
+        No new tasks will be accepted, enqueued ones will be performed.
+
+        """
+        self._context.state = CLOSED
+
+    def join(self, timeout=None):
+        """Joins the pool waiting until all workers exited.
+
+        If *timeout* is set, it block until all workers are done
+        or raise TimeoutError.
+
+        """
+        if self._context.state == RUNNING:
+            raise RuntimeError('The Pool is still running')
+        elif self._context.state == CLOSED:
+            queue = self._context.queue
+
+            if timeout is not None:
+                while queue.unfinished_tasks > 0 and timeout > 0:
+                    sleep(0.1)
+                    timeout -= 0.1
+            else:
+                queue.join()
+            self.stop()
+
+        self._context.join(timeout)
